@@ -13,12 +13,12 @@ spec :: Spec
 spec = do
   describe "Storable Complex instance" $ do
     it "has twice the sizeOf its realPart" $ do
-      sizeOf ((1 :: Double) :+ 2) `shouldBe` 2*sizeOf (1 :: Double)
+      sizeOf (undefined :: Complex Double) `shouldBe` 2*sizeOf (1 :: Double)
     it "has the alignment of its realPart" $ do
-      alignment ((1 :: Double) :+ 2) `shouldBe` alignment (1 :: Double)
+      alignment (undefined :: Complex Double) `shouldBe` alignment (1 :: Double)
 
   describe "Storable Ratio instance" $ do
     it "has twice the sizeOf its parameterized type" $ do
-      sizeOf ((1 :: Int) % 2) `shouldBe` 2*sizeOf (1 :: Int)
+      sizeOf (undefined :: Ratio Int) `shouldBe` 2*sizeOf (1 :: Int)
     it "has the alignment of its parameterized type" $ do
-      alignment ((1 :: Int) % 2) `shouldBe` alignment (1 :: Int)
+      alignment (undefined :: Ratio Int) `shouldBe` alignment (1 :: Int)
